@@ -1,5 +1,6 @@
 # Copyright 2015 - Mirantis, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -16,8 +17,6 @@
 from oslo_config import cfg
 from oslo_log import log as logging
 from pecan import rest
-import sqlalchemy as sa
-import tenacity
 from wsme import types as wtypes
 import wsmeext.pecan as wsme_pecan
 
@@ -52,11 +51,7 @@ def _load_deferred_output_field(action_ex):
 
 
 # Use retries to prevent possible failures.
-@tenacity.retry(
-    retry=tenacity.retry_if_exception_type(sa.exc.OperationalError),
-    stop=tenacity.stop_after_attempt(10),
-    wait=tenacity.wait_incrementing(increment=100)  # 0.1 seconds
-)
+@rest_utils.rest_retry_on_db_error
 def _get_action_execution(id):
     with db_api.transaction():
         return _get_action_execution_resource(db_api.get_action_execution(id))
@@ -150,16 +145,15 @@ class ActionExecutionsController(rest.RestController):
         :param action_ex: Action to execute
         """
         acl.enforce('action_executions:create', context.ctx())
-
         LOG.debug(
             "Create action_execution [action_execution=%s]",
             action_ex
         )
-
         name = action_ex.name
         description = action_ex.description or None
         action_input = action_ex.input or {}
         params = action_ex.params or {}
+        namespace = action_ex.workflow_namespace or ''
 
         if not name:
             raise exc.InputException(
@@ -170,6 +164,7 @@ class ActionExecutionsController(rest.RestController):
             name,
             action_input,
             description=description,
+            namespace=namespace,
             **params
         )
 
