@@ -3,6 +3,7 @@
 # Copyright 2015 Huawei Technologies Co., Ltd.
 # Copyright 2016 - Brocade Communications Systems, Inc.
 # Copyright 2018 - Extreme Networks, Inc.
+# Copyright 2019 - NetCracker Technology Corp.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -25,6 +26,7 @@ import wsmeext.pecan as wsme_pecan
 from mistral.api import access_control as acl
 from mistral.api.controllers.v2 import execution_report
 from mistral.api.controllers.v2 import resources
+from mistral.api.controllers.v2 import sub_execution
 from mistral.api.controllers.v2 import task
 from mistral.api.controllers.v2 import types
 from mistral import context
@@ -85,6 +87,7 @@ def _get_workflow_execution(id, must_exist=True):
 class ExecutionsController(rest.RestController):
     tasks = task.ExecutionTasksController()
     report = execution_report.ExecutionReportController()
+    executions = sub_execution.SubExecutionsController()
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Execution, wtypes.text)
@@ -331,17 +334,21 @@ class ExecutionsController(rest.RestController):
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Executions, types.uuid, int,
                          types.uniquelist, types.list, types.uniquelist,
-                         wtypes.text, types.uuid, wtypes.text, types.jsontype,
-                         types.uuid, types.uuid, STATE_TYPES, wtypes.text,
+                         wtypes.text, types.uuid, wtypes.text,
+                         types.uniquelist, types.jsontype, types.uuid,
+                         types.uuid, STATE_TYPES, wtypes.text,
                          types.jsontype, types.jsontype, wtypes.text,
-                         wtypes.text, bool, types.uuid, bool)
-    def get_all(self, marker=None, limit=None, sort_keys='created_at',
-                sort_dirs='asc', fields='', workflow_name=None,
-                workflow_id=None, description=None, params=None,
-                task_execution_id=None, root_execution_id=None, state=None,
-                state_info=None, input=None, output=None, created_at=None,
+                         wtypes.text, bool, types.uuid,
+                         bool, types.list)
+    def get_all(self, marker=None, limit=None,
+                sort_keys='created_at', sort_dirs='asc', fields='',
+                workflow_name=None, workflow_id=None, description=None,
+                tags=None, params=None, task_execution_id=None,
+                root_execution_id=None, state=None, state_info=None,
+                input=None, output=None, created_at=None,
                 updated_at=None, include_output=None, project_id=None,
-                all_projects=False):
+                all_projects=False, nulls=''):
+
         """Return all Executions.
 
         :param marker: Optional. Pagination marker for large data sets.
@@ -364,6 +371,7 @@ class ExecutionsController(rest.RestController):
                             workflow ID.
         :param description: Optional. Keep only resources with a specific
                             description.
+        :param tags: Optional. Keep only resources containing specific tags.
         :param params: Optional. Keep only resources with specific parameters.
         :param task_execution_id: Optional. Keep only resources with a
                                   specific task execution ID.
@@ -384,16 +392,22 @@ class ExecutionsController(rest.RestController):
             Admin required.
         :param all_projects: Optional. Get resources of all projects. Admin
             required.
+        :param nulls: Optional. The names of the columns with null value in
+                        the query.
         """
         acl.enforce('executions:list', context.ctx())
 
+        db_models.WorkflowExecution.check_allowed_none_values(nulls)
+
         if all_projects or project_id:
             acl.enforce('executions:list:all_projects', context.ctx())
 
         filters = filter_utils.create_filters_from_request_params(
+            none_values=nulls,
             created_at=created_at,
             workflow_name=workflow_name,
             workflow_id=workflow_id,
+            tags=tags,
             params=params,
             task_execution_id=task_execution_id,
             state=state,
