@@ -104,31 +104,40 @@ class WorkflowsController(rest.RestController, hooks.HookController):
 
         :param identifier: Optional. If provided, it's UUID of a workflow.
             Only one workflow can be updated with identifier param.
-        :param namespace: Optional. If provided int's the namespace of the
-                          workflow/workflows. currently namespace cannot be
+        :param namespace: Optional. If provided, it's the namespace of the
+                          workflow/workflows. Currently, namespace cannot be
                           changed.
 
-        The text is allowed to have definitions of multiple workflows. In this
-        case they all will be updated.
+        The text is allowed to have definitions of multiple workflows. In such
+        case, they all will be updated.
         """
         acl.enforce('workflows:update', context.ctx())
 
+        # NOTE(rakhmerov): We can't use normal method arguments to access
+        # request data because it will break dynamic sub controller lookup
+        # functionality (see _lookup() above) so we have to get the data
+        # directly from the request object.
+
         definition = pecan.request.text
         scope = pecan.request.GET.get('scope', 'private')
 
-        if scope not in resources.SCOPE_TYPES.values:
-            raise exc.InvalidModelException(
-                "Scope must be one of the following: %s; actual: "
-                "%s" % (resources.SCOPE_TYPES.values, scope)
-            )
+        # If "skip_validation" is present in the query string parameters
+        # then workflow language validation will be disabled.
+        skip_validation = 'skip_validation' in pecan.request.GET
+
+        resources.Workflow.validate_scope(scope)
+
+        if scope == 'public':
+            acl.enforce('workflows:publicize', context.ctx())
 
         LOG.debug("Update workflow(s) [definition=%s]", definition)
 
-        db_wfs = workflows.update_workflows(
+        db_wfs = rest_utils.rest_retry_on_db_error(workflows.update_workflows)(
             definition,
             scope=scope,
             identifier=identifier,
-            namespace=namespace
+            namespace=namespace,
+            validate=not skip_validation
         )
 
         workflow_list = [
@@ -143,31 +152,41 @@ class WorkflowsController(rest.RestController, hooks.HookController):
     def post(self, namespace=''):
         """Create a new workflow.
 
-        NOTE: The text is allowed to have definitions
-            of multiple workflows. In this case they all will be created.
-
         :param namespace: Optional. The namespace to create the workflow
             in. Workflows with the same name can be added to a given
-            project if are in two different namespaces.
+            project if they are in two different namespaces.
+
+        The text is allowed to have definitions of multiple workflows.
+        In such case, they all will be created.
         """
         acl.enforce('workflows:create', context.ctx())
 
+        # NOTE(rakhmerov): We can't use normal method arguments to access
+        # request data because it will break dynamic sub controller lookup
+        # functionality (see _lookup() above) so we have to get the data
+        # directly from the request object.
+
         definition = pecan.request.text
         scope = pecan.request.GET.get('scope', 'private')
+
+        # If "skip_validation" is present in the query string parameters
+        # then workflow language validation will be disabled.
+        skip_validation = 'skip_validation' in pecan.request.GET
+
         pecan.response.status = 201
 
-        if scope not in resources.SCOPE_TYPES.values:
-            raise exc.InvalidModelException(
-                "Scope must be one of the following: %s; actual: "
-                "%s" % (resources.SCOPE_TYPES.values, scope)
-            )
+        resources.Workflow.validate_scope(scope)
+
+        if scope == 'public':
+            acl.enforce('workflows:publicize', context.ctx())
 
         LOG.debug("Create workflow(s) [definition=%s]", definition)
 
-        db_wfs = workflows.create_workflows(
+        db_wfs = rest_utils.rest_retry_on_db_error(workflows.create_workflows)(
             definition,
             scope=scope,
-            namespace=namespace
+            namespace=namespace,
+            validate=not skip_validation
         )
 
         workflow_list = [
@@ -189,8 +208,12 @@ class WorkflowsController(rest.RestController, hooks.HookController):
         LOG.debug("Delete workflow [identifier=%s, namespace=%s]",
                   identifier, namespace)
 
-        with db_api.transaction():
-            db_api.delete_workflow_definition(identifier, namespace)
+        @rest_utils.rest_retry_on_db_error
+        def _delete_workflow_definition():
+            with db_api.transaction():
+                db_api.delete_workflow_definition(identifier, namespace)
+
+        _delete_workflow_definition()
 
     @rest_utils.wrap_wsme_controller_exception
     @wsme_pecan.wsexpose(resources.Workflows, types.uuid, int,
