@@ -1,5 +1,6 @@
 # Copyright 2015 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -31,7 +32,6 @@ from mistral import exceptions as exc
 from mistral.services import security
 from mistral_lib import utils
 
-
 # Definition objects.
 
 CONF = cfg.CONF
@@ -82,6 +82,19 @@ def validate_long_type_length(cls, field_name, value):
             raise exc.SizeLimitExceededException(msg)
 
 
+def validate_name_has_no_spaces(name):
+    """Makes sure name does not contain spaces."""
+    if name:
+        if " " in name:
+            msg = (
+                "Name '{}' must not contain spaces"
+            ).format(name)
+
+            LOG.error(msg)
+
+            raise exc.InvalidModelException(msg)
+
+
 def register_length_validator(attr_name):
     """Register an event listener on the attribute.
 
@@ -97,6 +110,20 @@ def register_length_validator(attr_name):
             )
 
 
+def register_name_validator():
+    """Register an event listener on the attribute.
+
+    This event listener will validate that name of object does not
+    contains spaces every time a 'set' occurs.
+    """
+    for cls in utils.iter_subclasses(Definition):
+        event.listen(
+            getattr(cls, "name"),
+            'set',
+            lambda t, v, o, i: validate_name_has_no_spaces(v)
+        )
+
+
 class Definition(mb.MistralSecureModelBase):
     __abstract__ = True
 
@@ -147,9 +174,12 @@ class ActionDefinition(Definition):
     """Contains info about registered Actions."""
 
     __tablename__ = 'action_definitions_v2'
-
+    namespace = sa.Column(sa.String(255), nullable=True)
     __table_args__ = (
-        sa.UniqueConstraint('name', 'project_id'),
+        sa.UniqueConstraint(
+            'name',
+            'namespace',
+            'project_id'),
         sa.Index('%s_is_system' % __tablename__, 'is_system'),
         sa.Index('%s_action_class' % __tablename__, 'action_class'),
         sa.Index('%s_project_id' % __tablename__, 'project_id'),
@@ -319,7 +349,6 @@ for cls in utils.iter_subclasses(Execution):
         retval=True
     )
 
-
 # Many-to-one for 'ActionExecution' and 'TaskExecution'.
 
 ActionExecution.task_execution_id = sa.Column(
@@ -378,7 +407,6 @@ WorkflowExecution.root_execution = relationship(
     lazy='select'
 )
 
-
 # Many-to-one for 'TaskExecution' and 'WorkflowExecution'.
 
 TaskExecution.workflow_execution_id = sa.Column(
@@ -550,6 +578,8 @@ mb.register_secure_model_hooks()
 for attr_name in ['input', 'output', 'params', 'published']:
     register_length_validator(attr_name)
 
+register_name_validator()
+
 
 class ResourceMember(mb.MistralModelBase):
     """Contains info about resource members."""
