@@ -37,7 +37,6 @@ from mistral.workflow import states
 from mistral_lib import actions as ml_actions
 from mistral_lib import utils
 
-
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
@@ -53,6 +52,7 @@ class Action(object):
     def __init__(self, action_def, action_ex=None, task_ex=None):
         self.action_def = action_def
         self.action_ex = action_ex
+        self.namespace = action_def.namespace if action_def else None
         self.task_ex = action_ex.task_execution if action_ex else task_ex
 
     @abc.abstractmethod
@@ -162,6 +162,7 @@ class Action(object):
             'state': states.RUNNING,
             'input': input_dict,
             'runtime_context': runtime_ctx,
+            'workflow_namespace': self.namespace,
             'description': desc,
             'is_sync': is_sync
         }
@@ -254,7 +255,7 @@ class PythonAction(Action):
             action_ex_id=action_ex_id
         )
 
-        execution_context = self._prepare_execution_context()
+        action_ex_ctx = self._prepare_execution_context()
 
         # Register an asynchronous command to send the action to
         # run on an executor outside of the main DB transaction.
@@ -267,7 +268,7 @@ class PythonAction(Action):
                 self.action_def.attributes or {},
                 self.action_ex.input,
                 self.action_ex.runtime_context.get('safe_rerun', False),
-                execution_context,
+                action_ex_ctx,
                 target=target,
                 timeout=timeout
             )
@@ -320,12 +321,14 @@ class PythonAction(Action):
         try:
             prepared_input_dict = self._prepare_input(input_dict)
 
-            a = a_m.get_action_class(self.action_def.name)(
+            a = a_m.get_action_class(self.action_def.name,
+                                     self.action_def.namespace)(
                 **prepared_input_dict
             )
 
             return a.is_sync()
         except BaseException as e:
+            LOG.exception(e)
             raise exc.InputException(str(e))
 
     def validate_input(self, input_dict):
@@ -393,7 +396,8 @@ class AdHocAction(PythonAction):
         self.action_spec = spec_parser.get_action_spec(action_def.spec)
 
         base_action_def = db_api.load_action_definition(
-            self.action_spec.get_base()
+            self.action_spec.get_base(),
+            namespace=action_def.namespace
         )
 
         if not base_action_def:
@@ -414,6 +418,7 @@ class AdHocAction(PythonAction):
         )
 
         self.adhoc_action_def = action_def
+        self.namespace = action_def.namespace
         self.task_ctx = task_ctx or {}
         self.wf_ctx = wf_ctx or {}
 
@@ -512,11 +517,13 @@ class AdHocAction(PythonAction):
 
     @profiler.trace('ad-hoc-action-gather-base-actions', hide_args=True)
     def _gather_base_actions(self, action_def, base_action_def):
-        """Find all base ad-hoc actions and store them
+        """Find all base ad-hoc actions and store them.
 
-        An ad-hoc action may be based on another ad-hoc action (and this
-        recursively). Using twice the same base action is not allowed to
-        avoid infinite loops. It stores the list of ad-hoc actions.
+        An ad-hoc action may be based on another ad-hoc action and this
+        works recursively, so that the base action can also be based on an
+        ad-hoc action. Using the same base action more than once in this
+        action hierarchy is not allowed to avoid infinite loops.
+        The method stores the list of ad-hoc actions.
 
         :param action_def: Action definition
         :type action_def: ActionDefinition
@@ -527,6 +534,7 @@ class AdHocAction(PythonAction):
         """
 
         self.adhoc_action_defs = [action_def]
+
         original_base_name = self.action_spec.get_name()
         action_names = set([original_base_name])
 
@@ -538,10 +546,12 @@ class AdHocAction(PythonAction):
 
             base_name = base.spec['base']
             try:
-                base = db_api.get_action_definition(base_name)
+                base = db_api.get_action_definition(base_name,
+                                                    namespace=base.namespace)
             except exc.DBEntityNotFoundError:
                 raise exc.InvalidActionException(
-                    "Failed to find action [action_name=%s]" % base_name
+                    "Failed to find action [action_name=%s namespace=%s] "
+                    % (base_name, base.namespace)
                 )
 
         # if the action is repeated
@@ -649,12 +659,13 @@ class WorkflowAction(Action):
 
 
 def resolve_action_definition(action_spec_name, wf_name=None,
-                              wf_spec_name=None):
+                              wf_spec_name=None, namespace=''):
     """Resolve action definition accounting for ad-hoc action namespacing.
 
     :param action_spec_name: Action name according to a spec.
     :param wf_name: Workflow name.
     :param wf_spec_name: Workflow name according to a spec.
+    :param namespace: The namespace of the action.
     :return: Action definition (python or ad-hoc).
     """
 
@@ -670,14 +681,17 @@ def resolve_action_definition(action_spec_name, wf_name=None,
 
         action_full_name = "%s.%s" % (wb_name, action_spec_name)
 
-        action_db = db_api.load_action_definition(action_full_name)
+        action_db = db_api.load_action_definition(action_full_name,
+                                                  namespace=namespace)
 
     if not action_db:
-        action_db = db_api.load_action_definition(action_spec_name)
+        action_db = db_api.load_action_definition(action_spec_name,
+                                                  namespace=namespace)
 
     if not action_db:
         raise exc.InvalidActionException(
-            "Failed to find action [action_name=%s]" % action_spec_name
+            "Failed to find action [action_name=%s] in [namespace=%s]" %
+            (action_spec_name, namespace)
         )
 
     return action_db
