@@ -1,5 +1,6 @@
 # Copyright 2014 - Mirantis, Inc.
 # Copyright 2017 - Brocade Communications Systems, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -19,8 +20,8 @@ import jsonschema
 import six
 
 from mistral import exceptions as exc
-from mistral.utils import inspect_utils
 from mistral.workflow import data_flow
+from mistral_lib.utils import inspect_utils
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -48,12 +49,13 @@ class Engine(object):
 
     @abc.abstractmethod
     def start_action(self, action_name, action_input,
-                     description=None, **params):
+                     description=None, namespace='', **params):
         """Starts the specific action.
 
         :param action_name: Action name.
         :param action_input: Action input data as a dictionary.
         :param description: Execution description.
+        :param namespace: The namespace of the action.
         :param params: Additional options for action running.
         :return: Action execution object.
         """
