@@ -22,12 +22,12 @@ import traceback as tb
 from mistral.db import utils as db_utils
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
-from mistral.engine import action_queue
+from mistral.engine import post_tx_queue
 from mistral.engine import tasks
 from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions as exc
 from mistral.lang import parser as spec_parser
-from mistral.services import scheduler
+from mistral.scheduler import base as sched_base
 from mistral.workflow import base as wf_base
 from mistral.workflow import commands as wf_cmds
 from mistral.workflow import states
@@ -60,6 +60,11 @@ def run_task(wf_cmd):
     task = _build_task_from_command(wf_cmd)
 
     try:
+        if task.waiting and task.rerun:
+            task.set_state(states.WAITING, 'Task is waiting.')
+
+            _schedule_refresh_task_state(task.task_ex.id)
+
         task.run()
     except exc.MistralException as e:
         wf_ex = wf_cmd.wf_ex
@@ -70,17 +75,23 @@ def run_task(wf_cmd):
             (e, wf_ex.name, task_spec.get_name(), tb.format_exc())
         )
 
-        LOG.error(msg)
-
-        task.set_state(states.ERROR, msg)
-
-        wf_handler.force_fail_workflow(wf_ex, msg)
+        force_fail_task(task.task_ex, msg, task=task)
 
         return
 
     _check_affected_tasks(task)
 
 
+def mark_task_running(task_ex, wf_spec):
+    task = _build_task_from_execution(wf_spec, task_ex)
+
+    old_task_state = task_ex.state
+
+    task.set_state(states.RUNNING, None, False)
+
+    task.notify(old_task_state, states.RUNNING)
+
+
 @profiler.trace('task-handler-on-action-complete', hide_args=True)
 def _on_action_complete(action_ex):
     """Handles action completion event.
@@ -114,11 +125,7 @@ def _on_action_complete(action_ex):
                " action=%s]:\n%s" %
                (e, wf_ex.name, task_ex.name, action_ex.name, tb.format_exc()))
 
-        LOG.error(msg)
-
-        task.set_state(states.ERROR, msg)
-
-        wf_handler.force_fail_workflow(wf_ex, msg)
+        force_fail_task(task_ex, msg, task=task)
 
         return
 
@@ -173,18 +180,14 @@ def _on_action_update(action_ex):
                " action=%s]:\n%s" %
                (e, wf_ex.name, task_ex.name, action_ex.name, tb.format_exc()))
 
-        LOG.error(msg)
-
-        task.set_state(states.ERROR, msg)
-
-        wf_handler.force_fail_workflow(wf_ex, msg)
+        force_fail_task(task_ex, msg, task=task)
 
         return
 
     _check_affected_tasks(task)
 
 
-def force_fail_task(task_ex, msg):
+def force_fail_task(task_ex, msg, task=None):
     """Forces the given task to fail.
 
     This method implements the 'forced' task fail without giving a chance
@@ -196,19 +199,31 @@ def force_fail_task(task_ex, msg):
 
     :param task_ex: Task execution.
     :param msg: Error message.
+    :param task: Task object. Optional.
     """
-    wf_spec = spec_parser.get_workflow_spec_by_execution_id(
-        task_ex.workflow_execution_id
-    )
 
-    task = _build_task_from_execution(wf_spec, task_ex)
+    LOG.error(msg)
 
+    if not task:
+        wf_spec = spec_parser.get_workflow_spec_by_execution_id(
+            task_ex.workflow_execution_id
+        )
+
+        task = _build_task_from_execution(wf_spec, task_ex)
+
+    old_task_state = task_ex.state
     task.set_state(states.ERROR, msg)
+    task.notify(old_task_state, states.ERROR)
+
+    task.save_finished_time()
 
     wf_handler.force_fail_workflow(task_ex.workflow_execution, msg)
 
 
 def continue_task(task_ex):
+    if not task_ex:
+        return
+
     wf_spec = spec_parser.get_workflow_spec_by_execution_id(
         task_ex.workflow_execution_id
     )
@@ -227,11 +242,7 @@ def continue_task(task_ex):
             (e, wf_ex.name, task_ex.name, tb.format_exc())
         )
 
-        LOG.error(msg)
-
-        task.set_state(states.ERROR, msg)
-
-        wf_handler.force_fail_workflow(wf_ex, msg)
+        force_fail_task(task_ex, msg, task=task)
 
         return
 
@@ -239,6 +250,9 @@ def continue_task(task_ex):
 
 
 def complete_task(task_ex, state, state_info):
+    if not task_ex:
+        return
+
     wf_spec = spec_parser.get_workflow_spec_by_execution_id(
         task_ex.workflow_execution_id
     )
@@ -255,17 +269,14 @@ def complete_task(task_ex, state, state_info):
             (e, wf_ex.name, task_ex.name, tb.format_exc())
         )
 
-        LOG.error(msg)
-
-        task.set_state(states.ERROR, msg)
-
-        wf_handler.force_fail_workflow(wf_ex, msg)
+        force_fail_task(task_ex, msg, task=task)
 
         return
 
     _check_affected_tasks(task)
 
 
+@profiler.trace('task-handler-check-affected-tasks', hide_args=True)
 def _check_affected_tasks(task):
     if not task.is_completed():
         return
@@ -287,8 +298,30 @@ def _check_affected_tasks(task):
         task_ex.name
     )
 
+    def _schedule_if_needed(t_ex_id):
+        # NOTE(rakhmerov): we need to minimize the number of scheduled jobs
+        # that refresh state of "join" tasks. We'll check if corresponding
+        # jobs are already scheduled. Note that we must ignore scheduled jobs
+        # that are currently being processed because of a possible race with
+        # the transaction that deletes scheduled jobs, i.e. the job may still
+        # exist in DB (the deleting transaction didn't commit yet) but it has
+        # already been processed and the task state hasn't changed.
+        sched = sched_base.get_system_scheduler()
+
+        jobs_exist = sched.has_scheduled_jobs(
+            key=_get_refresh_state_job_key(t_ex_id),
+            processing=False
+        )
+
+        if not jobs_exist:
+            _schedule_refresh_task_state(t_ex_id)
+
     for t_ex in affected_task_execs:
-        _schedule_refresh_task_state(t_ex)
+        post_tx_queue.register_operation(
+            _schedule_if_needed,
+            args=[t_ex.id],
+            in_tx=True
+        )
 
 
 def _build_task_from_execution(wf_spec, task_ex):
@@ -312,7 +345,8 @@ def _build_task_from_command(cmd):
             task_ex=cmd.task_ex,
             unique_key=cmd.task_ex.unique_key,
             waiting=cmd.task_ex.state == states.WAITING,
-            triggered_by=cmd.triggered_by
+            triggered_by=cmd.triggered_by,
+            rerun=cmd.rerun
         )
 
         if cmd.reset:
@@ -337,7 +371,8 @@ def _build_task_from_command(cmd):
 
 
 def _create_task(wf_ex, wf_spec, task_spec, ctx, task_ex=None,
-                 unique_key=None, waiting=False, triggered_by=None):
+                 unique_key=None, waiting=False, triggered_by=None,
+                 rerun=False):
     if task_spec.get_with_items():
         cls = tasks.WithItemsTask
     else:
@@ -351,12 +386,13 @@ def _create_task(wf_ex, wf_spec, task_spec, ctx, task_ex=None,
         task_ex=task_ex,
         unique_key=unique_key,
         waiting=waiting,
-        triggered_by=triggered_by
+        triggered_by=triggered_by,
+        rerun=rerun
     )
 
 
 @db_utils.retry_on_db_error
-@action_queue.process
+@post_tx_queue.run
 @profiler.trace('task-handler-refresh-task-state', hide_args=True)
 def _refresh_task_state(task_ex_id):
     with db_api.transaction():
@@ -365,6 +401,10 @@ def _refresh_task_state(task_ex_id):
         if not task_ex:
             return
 
+        if (states.is_completed(task_ex.state)
+                or task_ex.state == states.RUNNING):
+            return
+
         wf_ex = task_ex.workflow_execution
 
         if states.is_completed(wf_ex.state):
@@ -377,6 +417,10 @@ def _refresh_task_state(task_ex_id):
         wf_ctrl = wf_base.get_controller(wf_ex, wf_spec)
 
         with db_api.named_lock(task_ex.id):
+            # NOTE: we have to use this lock to prevent two (or more) such
+            # methods from changing task state and starting its action or
+            # workflow. Checking task state outside of this section is a
+            # performance optimization because locking is pretty expensive.
             db_api.refresh(task_ex)
 
             if (states.is_completed(task_ex.state)
@@ -411,7 +455,7 @@ def _refresh_task_state(task_ex_id):
                 )
 
 
-def _schedule_refresh_task_state(task_ex, delay=0):
+def _schedule_refresh_task_state(task_ex_id, delay=0):
     """Schedules task preconditions check.
 
     This method provides transactional decoupling of task preconditions
@@ -424,34 +468,37 @@ def _schedule_refresh_task_state(task_ex, delay=0):
     we'll have in this case (time between transactions) whereas scheduler
     is a special component that is designed to be resistant to failures.
 
-    :param task_ex: Task execution.
+    :param task_ex_id: Task execution ID.
     :param delay: Delay.
     """
-    key = _get_refresh_state_job_key(task_ex.id)
 
-    scheduler.schedule_call(
-        None,
-        _REFRESH_TASK_STATE_PATH,
-        delay,
-        key=key,
-        task_ex_id=task_ex.id
+    sched = sched_base.get_system_scheduler()
+
+    job = sched_base.SchedulerJob(
+        run_after=delay,
+        func_name=_REFRESH_TASK_STATE_PATH,
+        func_args={'task_ex_id': task_ex_id},
+        key=_get_refresh_state_job_key(task_ex_id)
     )
 
+    sched.schedule(job)
+
 
 def _get_refresh_state_job_key(task_ex_id):
     return 'th_r_t_s-%s' % task_ex_id
 
 
 @db_utils.retry_on_db_error
-@action_queue.process
+@post_tx_queue.run
 def _scheduled_on_action_complete(action_ex_id, wf_action):
     with db_api.transaction():
         if wf_action:
-            action_ex = db_api.get_workflow_execution(action_ex_id)
+            action_ex = db_api.load_workflow_execution(action_ex_id)
         else:
-            action_ex = db_api.get_action_execution(action_ex_id)
+            action_ex = db_api.load_action_execution(action_ex_id)
 
-        _on_action_complete(action_ex)
+        if action_ex:
+            _on_action_complete(action_ex)
 
 
 def schedule_on_action_complete(action_ex, delay=0):
@@ -476,28 +523,32 @@ def schedule_on_action_complete(action_ex, delay=0):
 
         return
 
-    key = 'th_on_a_c-%s' % action_ex.task_execution_id
+    sched = sched_base.get_system_scheduler()
 
-    scheduler.schedule_call(
-        None,
-        _SCHEDULED_ON_ACTION_COMPLETE_PATH,
-        delay,
-        key=key,
-        action_ex_id=action_ex.id,
-        wf_action=isinstance(action_ex, models.WorkflowExecution)
+    job = sched_base.SchedulerJob(
+        run_after=delay,
+        func_name=_SCHEDULED_ON_ACTION_COMPLETE_PATH,
+        func_args={
+            'action_ex_id': action_ex.id,
+            'wf_action': isinstance(action_ex, models.WorkflowExecution)
+        },
+        key='th_on_a_c-%s' % action_ex.task_execution_id
     )
 
+    sched.schedule(job)
+
 
 @db_utils.retry_on_db_error
-@action_queue.process
+@post_tx_queue.run
 def _scheduled_on_action_update(action_ex_id, wf_action):
     with db_api.transaction():
         if wf_action:
-            action_ex = db_api.get_workflow_execution(action_ex_id)
+            action_ex = db_api.load_workflow_execution(action_ex_id)
         else:
-            action_ex = db_api.get_action_execution(action_ex_id)
+            action_ex = db_api.load_action_execution(action_ex_id)
 
-        _on_action_update(action_ex)
+        if action_ex:
+            _on_action_update(action_ex)
 
 
 def schedule_on_action_update(action_ex, delay=0):
@@ -522,13 +573,16 @@ def schedule_on_action_update(action_ex, delay=0):
 
         return
 
-    key = 'th_on_a_u-%s' % action_ex.task_execution_id
+    sched = sched_base.get_system_scheduler()
 
-    scheduler.schedule_call(
-        None,
-        _SCHEDULED_ON_ACTION_UPDATE_PATH,
-        delay,
-        key=key,
-        action_ex_id=action_ex.id,
-        wf_action=isinstance(action_ex, models.WorkflowExecution)
+    job = sched_base.SchedulerJob(
+        run_after=delay,
+        func_name=_SCHEDULED_ON_ACTION_UPDATE_PATH,
+        func_args={
+            'action_ex_id': action_ex.id,
+            'wf_action': isinstance(action_ex, models.WorkflowExecution)
+        },
+        key='th_on_a_u-%s' % action_ex.task_execution_id
     )
+
+    sched.schedule(job)
