@@ -14,25 +14,72 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+import collections
 import inspect
 import re
 
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 import six
+from yaml import representer
+import yaql
 from yaql.language import exceptions as yaql_exc
 from yaql.language import factory
+from yaql.language import utils as yaql_utils
 
 from mistral.config import cfg
 from mistral import exceptions as exc
-from mistral.expressions.base_expression import Evaluator
-from mistral import utils
-from mistral.utils import expression_utils
+from mistral.expressions import base
+from mistral_lib import utils
+
 
 LOG = logging.getLogger(__name__)
 
 _YAQL_CONF = cfg.CONF.yaql
 
+INLINE_YAQL_REGEXP = '<%.*?%>'
+
+YAQL_ENGINE = None
+
+ROOT_YAQL_CONTEXT = None
+
+# TODO(rakhmerov): it's work around the bug in YAQL.
+# YAQL shouldn't expose internal types to custom functions.
+representer.SafeRepresenter.add_representer(
+    yaql_utils.FrozenDict,
+    representer.SafeRepresenter.represent_dict
+)
+
+
+def get_yaql_context(data_context):
+    global ROOT_YAQL_CONTEXT
+
+    if not ROOT_YAQL_CONTEXT:
+        ROOT_YAQL_CONTEXT = yaql.create_context()
+
+        _register_yaql_functions(ROOT_YAQL_CONTEXT)
+
+    new_ctx = ROOT_YAQL_CONTEXT.create_child_context()
+
+    new_ctx['$'] = (
+        data_context if not cfg.CONF.yaql.convert_input_data
+        else yaql_utils.convert_input_data(data_context)
+    )
+
+    if isinstance(data_context, dict):
+        new_ctx['__env'] = data_context.get('__env')
+        new_ctx['__execution'] = data_context.get('__execution')
+        new_ctx['__task_execution'] = data_context.get('__task_execution')
+
+    return new_ctx
+
+
+def _register_yaql_functions(yaql_ctx):
+    functions = base.get_custom_functions()
+
+    for name in functions:
+        yaql_ctx.register_function(functions[name], name=name)
+
 
 def get_yaql_engine_options():
     return {
@@ -41,7 +88,7 @@ def get_yaql_engine_options():
         "yaql.convertTuplesToLists": _YAQL_CONF.convert_tuples_to_lists,
         "yaql.convertSetsToLists": _YAQL_CONF.convert_sets_to_lists,
         "yaql.iterableDicts": _YAQL_CONF.iterable_dicts,
-        "yaql.convertOutputData": True
+        "yaql.convertOutputData": _YAQL_CONF.convert_output_data
     }
 
 
@@ -53,31 +100,53 @@ def create_yaql_engine_class(keyword_operator, allow_delegates,
     ).create(options=engine_options)
 
 
-YAQL_ENGINE = create_yaql_engine_class(
-    _YAQL_CONF.keyword_operator,
-    _YAQL_CONF.allow_delegates,
-    get_yaql_engine_options()
-)
+def get_yaql_engine_class():
+    global YAQL_ENGINE
 
-LOG.info(
-    "YAQL engine has been initialized with the options: \n%s",
-    utils.merge_dicts(
-        get_yaql_engine_options(),
-        {
-            "keyword_operator": _YAQL_CONF.keyword_operator,
-            "allow_delegates": _YAQL_CONF.allow_delegates
-        }
+    if YAQL_ENGINE is not None:
+        return YAQL_ENGINE
+
+    YAQL_ENGINE = create_yaql_engine_class(
+        _YAQL_CONF.keyword_operator,
+        _YAQL_CONF.allow_delegates,
+        get_yaql_engine_options()
     )
-)
 
-INLINE_YAQL_REGEXP = '<%.*?%>'
+    LOG.info(
+        "YAQL engine has been initialized with the options: \n%s",
+        utils.merge_dicts(
+            get_yaql_engine_options(),
+            {
+                "keyword_operator": _YAQL_CONF.keyword_operator,
+                "allow_delegates": _YAQL_CONF.allow_delegates
+            }
+        )
+    )
+
+    return YAQL_ENGINE
 
 
-class YAQLEvaluator(Evaluator):
+def _sanitize_yaql_result(result):
+    # Expression output conversion can be disabled but we can still
+    # do some basic unboxing if we got an internal YAQL type.
+    # TODO(rakhmerov): FrozenDict doesn't provide any public method
+    # or property to access a regular dict that it wraps so ideally
+    # we need to add it to YAQL. Once it's there we need to make a
+    # fix here.
+    if isinstance(result, yaql_utils.FrozenDict):
+        return result._d
+
+    if inspect.isgenerator(result) or isinstance(result, collections.Iterator):
+        return list(result)
+
+    return result
+
+
+class YAQLEvaluator(base.Evaluator):
     @classmethod
     def validate(cls, expression):
         try:
-            YAQL_ENGINE(expression)
+            get_yaql_engine_class()(expression)
         except (yaql_exc.YaqlException, KeyError, ValueError, TypeError) as e:
             raise exc.YaqlGrammarException(getattr(e, 'message', e))
 
@@ -86,8 +155,8 @@ class YAQLEvaluator(Evaluator):
         expression = expression.strip() if expression else expression
 
         try:
-            result = YAQL_ENGINE(expression).evaluate(
-                context=expression_utils.get_yaql_context(data_context)
+            result = get_yaql_engine_class()(expression).evaluate(
+                context=get_yaql_context(data_context)
             )
         except Exception as e:
             # NOTE(rakhmerov): if we hit a database error then we need to
@@ -113,7 +182,7 @@ class YAQLEvaluator(Evaluator):
                 ", data=%s]" % (expression, str(e), data_context)
             )
 
-        return result if not inspect.isgenerator(result) else list(result)
+        return _sanitize_yaql_result(result)
 
     @classmethod
     def is_expression(cls, s):
@@ -143,10 +212,8 @@ class InlineYAQLEvaluator(YAQLEvaluator):
     @classmethod
     def evaluate(cls, expression, data_context):
         LOG.debug(
-            "Start to evaluate YAQL expression. "
-            "[expression='%s', context=%s]",
-            expression,
-            data_context
+            "Starting to evaluate YAQL expression. "
+            "[expression='%s']", expression
         )
 
         result = expression
@@ -155,19 +222,17 @@ class InlineYAQLEvaluator(YAQLEvaluator):
         if found_expressions:
             for expr in found_expressions:
                 trim_expr = expr.strip("<%>")
-                evaluated = super(InlineYAQLEvaluator,
-                                  cls).evaluate(trim_expr, data_context)
+
+                evaluated = super(InlineYAQLEvaluator, cls).evaluate(
+                    trim_expr,
+                    data_context
+                )
+
                 if len(expression) == len(expr):
                     result = evaluated
                 else:
                     result = result.replace(expr, str(evaluated))
 
-        LOG.debug(
-            "Finished evaluation. [expression='%s', result: %s]",
-            expression,
-            result
-        )
-
         return result
 
     @classmethod
