@@ -51,13 +51,13 @@ class RetrySpec(base.BaseSpec):
     }
 
     @classmethod
-    def get_schema(cls, includes=['definitions']):
+    def get_schema(cls, includes=('definitions',)):
         return super(RetrySpec, cls).get_schema(includes)
 
-    def __init__(self, data):
+    def __init__(self, data, validate):
         data = self._transform_retry_one_line(data)
 
-        super(RetrySpec, self).__init__(data)
+        super(RetrySpec, self).__init__(data, validate)
 
         self._break_on = data.get('break-on')
         self._count = data.get('count')
