@@ -19,6 +19,7 @@ from mistral.lang import types
 from mistral.lang.v2 import base
 from mistral.lang.v2 import on_clause
 from mistral.lang.v2 import policies
+from mistral.lang.v2 import retry_policy
 
 
 # TODO(rakhmerov): This specification should be broken into two separate
@@ -31,15 +32,17 @@ class TaskDefaultsSpec(base.BaseSpec):
     _schema = {
         "type": "object",
         "properties": {
-            "retry": types.ANY,
-            "wait-before": types.ANY,
-            "wait-after": types.ANY,
-            "timeout": types.ANY,
-            "pause-before": types.ANY,
-            "concurrency": types.ANY,
-            "on-complete": types.ANY,
-            "on-success": types.ANY,
-            "on-error": types.ANY,
+            "retry": retry_policy.RetrySpec.get_schema(),
+            "wait-before": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "wait-after": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "timeout": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "pause-before": types.EXPRESSION_OR_BOOLEAN,
+            "concurrency": types.EXPRESSION_OR_POSITIVE_INTEGER,
+            "fail-on": types.EXPRESSION_OR_BOOLEAN,
+            "on-complete": on_clause.OnClauseSpec.get_schema(),
+            "on-success": on_clause.OnClauseSpec.get_schema(),
+            "on-error": on_clause.OnClauseSpec.get_schema(),
+            "safe-rerun": types.EXPRESSION_OR_BOOLEAN,
             "requires": {
                 "oneOf": [types.NONEMPTY_STRING, types.UNIQUE_STRING_LIST]
             }
@@ -48,11 +51,11 @@ class TaskDefaultsSpec(base.BaseSpec):
     }
 
     @classmethod
-    def get_schema(cls, includes=['definitions']):
+    def get_schema(cls, includes=('definitions',)):
         return super(TaskDefaultsSpec, cls).get_schema(includes)
 
-    def __init__(self, data):
-        super(TaskDefaultsSpec, self).__init__(data)
+    def __init__(self, data, validate):
+        super(TaskDefaultsSpec, self).__init__(data, validate)
 
         self._policies = self._group_spec(
             policies.PoliciesSpec,
@@ -61,7 +64,8 @@ class TaskDefaultsSpec(base.BaseSpec):
             'wait-after',
             'timeout',
             'pause-before',
-            'concurrency'
+            'concurrency',
+            'fail-on'
         )
 
         on_spec_cls = on_clause.OnClauseSpec
@@ -70,10 +74,17 @@ class TaskDefaultsSpec(base.BaseSpec):
         self._on_success = self._spec_property('on-success', on_spec_cls)
         self._on_error = self._spec_property('on-error', on_spec_cls)
 
+        self._safe_rerun = data.get('safe-rerun')
+
         # TODO(rakhmerov): 'requires' should reside in a different spec for
         # reverse workflows.
         self._requires = data.get('requires', [])
 
+    def validate_schema(self):
+        super(TaskDefaultsSpec, self).validate_schema()
+
+        self.validate_expr(self._data.get('safe-rerun', {}))
+
     def validate_semantics(self):
         # Validate YAQL expressions.
         self._validate_transitions(self._on_complete)
@@ -101,6 +112,9 @@ class TaskDefaultsSpec(base.BaseSpec):
     def get_on_error(self):
         return self._on_error
 
+    def get_safe_rerun(self):
+        return self._safe_rerun
+
     def get_requires(self):
         if isinstance(self._requires, six.string_types):
             return [self._requires]
