@@ -13,15 +13,15 @@
 #    limitations under the License.
 
 import copy
+from unittest import mock
 
-import mock
 import sqlalchemy as sa
 
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models
 from mistral import exceptions as exc
 from mistral.tests.unit.api import base
-from mistral import utils
+from mistral_lib import utils
 
 
 ACTION_DEFINITION = """
