@@ -19,8 +19,8 @@
 import copy
 import datetime
 import json
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 import oslo_messaging
 from oslo_utils import uuidutils
@@ -28,6 +28,7 @@ import sqlalchemy as sa
 from webtest import app as webtest_app
 
 from mistral.api.controllers.v2 import execution
+from mistral.api.controllers.v2 import resources
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import api as sql_db_api
 from mistral.db.v2.sqlalchemy import models
@@ -903,7 +904,8 @@ class TestExecutionsController(base.APITest):
         )
 
     @mock.patch.object(db_api, 'get_workflow_executions', MOCK_WF_EXECUTIONS)
-    @mock.patch.object(rest_utils, 'get_all')
+    @mock.patch.object(rest_utils, 'get_all',
+                       return_value=resources.Executions())
     def test_get_all_executions_with_output(self, mock_get_all):
         resp = self.app.get('/v2/executions?include_output=true')
 
@@ -918,7 +920,8 @@ class TestExecutionsController(base.APITest):
         )
 
     @mock.patch.object(db_api, 'get_workflow_executions', MOCK_WF_EXECUTIONS)
-    @mock.patch.object(rest_utils, 'get_all')
+    @mock.patch.object(rest_utils, 'get_all',
+                       return_value=resources.Executions())
     def test_get_all_executions_without_output(self, mock_get_all):
         resp = self.app.get('/v2/executions')
 
@@ -968,3 +971,26 @@ class TestExecutionsController(base.APITest):
         self.assertTrue(
             mock_get_execs.call_args[1].get('project_id', fake_project_id)
         )
+
+    def test_get_all_with_nulls_not_valid(self):
+        resp = self.app.get(
+            '/v2/executions?limit=10&sort_keys=id&sort_dirs=asc&nulls=invalid',
+            expect_errors=True
+        )
+
+        self.assertEqual(500, resp.status_int)
+        self.assertIn(
+            "'invalid' is not a valid field name.",
+            resp.body.decode()
+        )
+
+        resp = self.app.get(
+            '/v2/executions?limit=10&sort_keys=id&sort_dirs=asc&nulls=id',
+            expect_errors=True
+        )
+
+        self.assertEqual(500, resp.status_int)
+        self.assertIn(
+            "The field 'id' can't hold None value.",
+            resp.body.decode()
+        )
