@@ -14,9 +14,8 @@
 #    limitations under the License.
 
 import copy
-import datetime
+from unittest import mock
 
-import mock
 import sqlalchemy as sa
 
 from mistral.db.v2 import api as db_api
@@ -27,29 +26,45 @@ from mistral.tests.unit.api import base
 
 WORKBOOK_DEF = """
 ---
-version: 2.0
+version: '2.0'
 name: 'test'
+
+workflows:
+  flow:
+    type: direct
+    tasks:
+      task1:
+        action: step param="Hello !"
+
+actions:
+  step:
+    input:
+      - param
+    base: std.echo output="<% $.param %>""
 """
 
 UPDATED_WORKBOOK_DEF = """
 ---
-version: 2.0
+version: '2.0'
 name: 'book'
-"""
 
-WORKBOOK_DB = models.Workbook(
-    id='123',
-    name='book',
-    definition=WORKBOOK_DEF,
-    tags=['deployment', 'demo'],
-    scope="public",
-    created_at=datetime.datetime(1970, 1, 1),
-    updated_at=datetime.datetime(1970, 1, 1)
-)
+workflows:
+  flow:
+    type: direct
+    tasks:
+      task1:
+        action: step arg="Hello !"
+
+actions:
+  step:
+    input:
+      - param
+    base: std.echo output="<% $.param %>""
+"""
 
 WORKBOOK = {
     'id': '123',
-    'name': 'book',
+    'name': 'test',
     'definition': WORKBOOK_DEF,
     'tags': ['deployment', 'demo'],
     'scope': 'public',
@@ -57,6 +72,45 @@ WORKBOOK = {
     'updated_at': '1970-01-01 00:00:00'
 }
 
+WB_WITH_NAMESPACE = {
+    'id': '123',
+    'name': 'test',
+    'namespace': 'xyz',
+    'definition': WORKBOOK_DEF,
+    'tags': ['deployment', 'demo'],
+    'scope': 'public',
+    'created_at': '1970-01-01 00:00:00',
+    'updated_at': '1970-01-01 00:00:00'
+}
+
+ACTION = {
+    'id': '123e4567-e89b-12d3-a456-426655440000',
+    'name': 'step',
+    'is_system': False,
+    'description': 'My super cool action.',
+    'tags': ['test', 'v2'],
+    'definition': ''
+}
+
+WF = {
+    'id': '123e4567-e89b-12d3-a456-426655440000',
+    'name': 'flow',
+    'definition': '',
+    'created_at': '1970-01-01 00:00:00',
+    'updated_at': '1970-01-01 00:00:00',
+}
+
+ACTION_DB = models.ActionDefinition()
+ACTION_DB.update(ACTION)
+
+WORKBOOK_DB = models.Workbook()
+WORKBOOK_DB.update(WORKBOOK)
+
+WB_DB_WITH_NAMESPACE = models.Workbook(**WB_WITH_NAMESPACE)
+
+WF_DB = models.WorkflowDefinition()
+WF_DB.update(WF)
+
 WORKBOOK_DB_PROJECT_ID = WORKBOOK_DB.get_clone()
 WORKBOOK_DB_PROJECT_ID.project_id = '<default-project>'
 
@@ -98,6 +152,7 @@ workflows:
 """
 
 MOCK_WORKBOOK = mock.MagicMock(return_value=WORKBOOK_DB)
+MOCK_WB_WITH_NAMESPACE = mock.MagicMock(return_value=WB_DB_WITH_NAMESPACE)
 MOCK_WORKBOOKS = mock.MagicMock(return_value=[WORKBOOK_DB])
 MOCK_UPDATED_WORKBOOK = mock.MagicMock(return_value=UPDATED_WORKBOOK_DB)
 MOCK_DELETE = mock.MagicMock(return_value=None)
@@ -114,6 +169,13 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(200, resp.status_int)
         self.assertDictEqual(WORKBOOK, resp.json)
 
+    @mock.patch.object(db_api, "get_workbook", MOCK_WB_WITH_NAMESPACE)
+    def test_get_with_namespace(self):
+        resp = self.app.get('/v2/workbooks/123?namespace=xyz')
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(WB_WITH_NAMESPACE, resp.json)
+
     @mock.patch.object(db_api, 'get_workbook')
     def test_get_operational_error(self, mocked_get):
         mocked_get.side_effect = [
@@ -174,6 +236,51 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    @mock.patch.object(workbooks, "update_workbook_v2", MOCK_UPDATED_WORKBOOK)
+    def test_put_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.put(
+            '/v2/workbooks?skip_validation',
+            WB_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(200, resp.status_int)
+
+    @mock.patch.object(db_api, "update_workbook")
+    @mock.patch.object(db_api, "create_or_update_workflow_definition")
+    @mock.patch.object(db_api, "create_or_update_action_definition")
+    def test_put_public(self, mock_action, mock_wf, mock_wb):
+        mock_wb.return_value = UPDATED_WORKBOOK_DB
+        mock_wf.return_value = WF_DB
+        mock_action.return_value = ACTION_DB
+
+        resp = self.app.put(
+            '/v2/workbooks?scope=public',
+            UPDATED_WORKBOOK_DEF,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(200, resp.status_int)
+        self.assertDictEqual(UPDATED_WORKBOOK, resp.json)
+
+        self.assertEqual("public", mock_wb.call_args[0][1]['scope'])
+        self.assertEqual("public", mock_wf.call_args[0][1]['scope'])
+        self.assertEqual("public", mock_action.call_args[0][1]['scope'])
+
+    def test_put_wrong_scope(self):
+        resp = self.app.put(
+            '/v2/workbooks?scope=unique',
+            UPDATED_WORKBOOK_DEF,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(400, resp.status_int)
+        self.assertIn("Scope must be one of the following", resp.body.decode())
+
     @mock.patch.object(workbooks, "create_workbook_v2", MOCK_WORKBOOK)
     def test_post(self):
         resp = self.app.post(
@@ -185,6 +292,19 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(201, resp.status_int)
         self.assertEqual(WORKBOOK, resp.json)
 
+    @mock.patch.object(workbooks, "create_workbook_v2", MOCK_WB_WITH_NAMESPACE)
+    def test_post_namespace(self):
+
+        namespace = 'xyz'
+        resp = self.app.post(
+            '/v2/workbooks?namespace=%s' % namespace,
+            WORKBOOK_DEF,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(201, resp.status_int)
+        self.assertEqual(WB_WITH_NAMESPACE, resp.json)
+
     @mock.patch.object(workbooks, "create_workbook_v2", MOCK_DUPLICATE)
     def test_post_dup(self):
         resp = self.app.post(
@@ -207,6 +327,50 @@ class TestWorkbooksController(base.APITest):
         self.assertEqual(400, resp.status_int)
         self.assertIn("Invalid DSL", resp.body.decode())
 
+    def test_post_invalid_skip_validation(self):
+        self.override_config('validation_mode', 'enabled', 'api')
+
+        resp = self.app.post(
+            '/v2/workbooks?skip_validation',
+            WB_DEF_INVALID_MODEL_EXCEPTION,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(201, resp.status_int)
+
+    @mock.patch.object(db_api, "create_workbook")
+    @mock.patch.object(db_api, "create_or_update_workflow_definition")
+    @mock.patch.object(db_api, "create_or_update_action_definition")
+    def test_post_public(self, mock_action, mock_wf, mock_wb):
+        mock_wb.return_value = WORKBOOK_DB
+        mock_wf.return_value = WF_DB
+        mock_action.return_value = ACTION_DB
+
+        resp = self.app.post(
+            '/v2/workbooks?scope=public',
+            WORKBOOK_DEF,
+            headers={'Content-Type': 'text/plain'}
+        )
+
+        self.assertEqual(201, resp.status_int)
+        self.assertEqual(WORKBOOK, resp.json)
+
+        self.assertEqual("public", mock_wb.call_args[0][0]['scope'])
+        self.assertEqual("public", mock_wf.call_args[0][1]['scope'])
+        self.assertEqual("public", mock_action.call_args[0][1]['scope'])
+
+    def test_post_wrong_scope(self):
+        resp = self.app.post(
+            '/v2/workbooks?scope=unique',
+            WORKBOOK_DEF,
+            headers={'Content-Type': 'text/plain'},
+            expect_errors=True
+        )
+
+        self.assertEqual(400, resp.status_int)
+        self.assertIn("Scope must be one of the following", resp.body.decode())
+
     @mock.patch.object(db_api, "delete_workbook", MOCK_DELETE)
     def test_delete(self):
         resp = self.app.delete('/v2/workbooks/123')
