@@ -18,14 +18,13 @@ import json
 import pkg_resources as pkg
 import sys
 import time
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslotest import base
 import testtools.matchers as ttm
 
-from mistral import config
 from mistral import context as auth_context
 from mistral.db.sqlalchemy import base as db_sa_base
 from mistral.db.sqlalchemy import sqlite_lock
@@ -34,9 +33,8 @@ from mistral.lang import parser as spec_parser
 from mistral.services import action_manager
 from mistral.services import security
 from mistral.tests.unit import config as test_config
-from mistral.utils import inspect_utils as i_utils
 from mistral import version
-from mistral.workflow import lookup_utils
+from mistral_lib.utils import inspect_utils as i_utils
 
 RESOURCES_PATH = 'tests/resources/'
 LOG = logging.getLogger(__name__)
@@ -140,6 +138,9 @@ class BaseTest(base.BaseTestCase):
     def _assert_single_item(self, items, **props):
         return self._assert_multiple_items(items, 1, **props)[0]
 
+    def _assert_no_item(self, items, **props):
+        self._assert_multiple_items(items, 0, **props)
+
     def _assert_multiple_items(self, items, count, **props):
         def _matches(item, **props):
             for prop_name, prop_val in props.items():
@@ -263,14 +264,6 @@ class DbTestCase(BaseTest):
         if cfg.CONF.database.connection.startswith('sqlite'):
             cfg.CONF.set_default('connection', 'sqlite://', group='database')
 
-        # This option is normally registered in sync_db.py so we have to
-        # register it here specifically for tests.
-        cfg.CONF.register_opt(config.os_actions_mapping_path)
-
-        cfg.CONF.set_default(
-            'openstack_actions_mapping_path',
-            'tests/resources/openstack/test_mapping.json'
-        )
         cfg.CONF.set_default('max_overflow', -1, group='database')
         cfg.CONF.set_default('max_pool_size', 1000, group='database')
 
@@ -279,7 +272,7 @@ class DbTestCase(BaseTest):
         action_manager.sync_db()
 
     def _clean_db(self):
-        lookup_utils.clear_caches()
+        db_api._ACTION_DEF_CACHE.clear()
 
         contexts = [
             get_context(default=False),
