@@ -1,4 +1,5 @@
 # Copyright 2014 - Mirantis, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -15,6 +16,7 @@
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
+from mistral import exceptions as exc
 from mistral.services import workbooks as wb_service
 from mistral.tests.unit.engine import base
 from mistral.workflow import states
@@ -24,8 +26,7 @@ from mistral.workflow import states
 # the change in value is not permanent.
 cfg.CONF.set_default('auth_enable', False, group='pecan')
 
-
-WORKBOOK = """
+WB = """
 ---
 version: '2.0'
 
@@ -161,7 +162,7 @@ class AdhocActionsTest(base.EngineTestCase):
     def setUp(self):
         super(AdhocActionsTest, self).setUp()
 
-        wb_service.create_workbook_v2(WORKBOOK)
+        wb_service.create_workbook_v2(WB)
 
     def test_run_workflow_with_adhoc_action(self):
         wf_ex = self.engine.start_workflow(
@@ -287,10 +288,7 @@ class AdhocActionsTest(base.EngineTestCase):
             input:
               - my_param
 
-            base: std.mistral_http
-            base-input:
-              url: http://google.com/<% $.my_param %>
-              method: GET
+            base: std.async_noop
 
         workflows:
           my_wf:
@@ -305,6 +303,96 @@ class AdhocActionsTest(base.EngineTestCase):
 
         self.await_workflow_running(wf_ex.id)
 
+    def test_adhoc_action_definition_with_namespace(self):
+        namespace1 = 'ad-hoc_test'
+        namespace2 = 'ad-hoc_test2'
+
+        wb_text = """---
+
+        version: '2.0'
+
+        name: my_wb1
+
+        actions:
+          test_env:
+            base: std.echo
+            base-input:
+              output: '{{ env().foo }}' # TODO(rakhmerov): It won't work.
+
+        workflows:
+          wf:
+            input:
+              - str1
+            output:
+                workflow_result: '{{ _.printenv_result }}'
+
+            tasks:
+              printenv:
+                action: test_env
+                publish:
+                  printenv_result: '{{ task().result }}'
+        """
+
+        wb_service.create_workbook_v2(wb_text, namespace=namespace1)
+        wb_service.create_workbook_v2(wb_text, namespace=namespace2)
+
+        with db_api.transaction():
+            action_defs = db_api.get_action_definitions(
+                name='my_wb1.test_env'
+            )
+
+            self.assertEqual(2, len(action_defs))
+
+            action_defs = db_api.get_action_definitions(
+                name='my_wb1.test_env',
+                namespace=namespace1
+            )
+
+            self.assertEqual(1, len(action_defs))
+
+            action_defs = db_api.get_action_definitions(
+                name='my_wb1.test_env',
+                namespace=namespace2
+            )
+
+            self.assertEqual(1, len(action_defs))
+
+            self.assertRaises(
+                exc.DBEntityNotFoundError,
+                db_api.get_action_definition,
+                name='my_wb1.test_env'
+            )
+
+    def test_adhoc_action_execution_with_namespace(self):
+        namespace = 'ad-hoc_test'
+
+        wb_service.create_workbook_v2(WB, namespace=namespace)
+
+        wf_ex = self.engine.start_workflow(
+            'my_wb.wf4',
+            wf_input={'str1': 'a'},
+            env={'foo': 'bar'},
+            wf_namespace=namespace
+        )
+
+        self.await_workflow_success(wf_ex.id)
+
+        with db_api.transaction():
+            action_execs = db_api.get_action_executions(
+                name='std.echo',
+                workflow_namespace=namespace
+            )
+
+            self.assertEqual(1, len(action_execs))
+
+            context = action_execs[0].runtime_context
+
+            self.assertEqual(
+                'my_wb.test_env',
+                context.get('adhoc_action_name')
+            )
+            self.assertEqual(namespace, action_execs[0].workflow_namespace)
+
     def test_adhoc_action_runtime_context_name(self):
         wf_ex = self.engine.start_workflow(
             'my_wb.wf4',
@@ -316,6 +404,7 @@ class AdhocActionsTest(base.EngineTestCase):
 
         with db_api.transaction():
             action_execs = db_api.get_action_executions(name='std.echo')
+
             self.assertEqual(1, len(action_execs))
 
             action_name = action_execs[0].runtime_context.get(
