@@ -12,7 +12,7 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-import mock
+from unittest import mock
 
 from oslo_config import cfg
 
@@ -63,7 +63,7 @@ class ReverseWorkflowRerunCancelledTest(base.EngineTestCase):
 
         wb_service.create_workbook_v2(wb_def)
 
-        wf1_ex = self.engine.start_workflow('wb1.wf1', '', {}, task_name='t3')
+        wf1_ex = self.engine.start_workflow('wb1.wf1', task_name='t3')
 
         self.await_workflow_state(wf1_ex.id, states.RUNNING)
 
@@ -137,11 +137,14 @@ class ReverseWorkflowRerunCancelledTest(base.EngineTestCase):
         self.assertEqual(2, len(wf1_t1_action_exs))
         # Check there is exactly 1 action in Running and 1 in Cancelled state.
         # Order doesn't matter.
-        self._assert_single_item(wf1_t1_action_exs, state=states.RUNNING)
         self._assert_single_item(wf1_t1_action_exs, state=states.CANCELLED)
+        running_execution = self._assert_single_item(
+            wf1_t1_action_exs,
+            state=states.RUNNING
+        )
 
         self.engine.on_action_complete(
-            wf1_t1_action_exs[1].id,
+            running_execution.id,
             ml_actions.Result(data={'foo': 'bar'})
         )
 
