@@ -21,44 +21,46 @@ from mistral_lib import actions as ml_actions
 
 
 class SubworkflowPauseResumeTest(base.EngineTestCase):
-
     def test_pause_resume_cascade_down_to_subworkflow(self):
-        workbook = """
+        wb_text = """
         version: '2.0'
+
         name: wb
+
         workflows:
           wf1:
             tasks:
               task1:
                 workflow: wf2
-                on-success:
-                  - task3
+                on-success: task3
+
               task2:
                 workflow: wf3
-                on-success:
-                  - task3
+                on-success: task3
+
               task3:
                 join: all
-                action: std.noop
+
           wf2:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
+
           wf3:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
         """
 
-        wb_service.create_workbook_v2(workbook)
+        wb_service.create_workbook_v2(wb_text)
 
         # Start workflow execution.
         wf_1_ex = self.engine.start_workflow('wb.wf1')
@@ -148,8 +150,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -167,8 +167,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -180,8 +178,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -216,8 +212,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -235,8 +229,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -248,8 +240,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -323,10 +313,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 name='task1'
             )
 
-            wf_2_task_1_action_exs = db_api.get_action_executions(
-                task_execution_id=wf_2_task_1_ex.id
-            )
-
             wf_2_task_2_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task2'
@@ -341,10 +327,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 name='task1'
             )
 
-            wf_3_task_1_action_exs = db_api.get_action_executions(
-                task_execution_id=wf_3_task_1_ex.id
-            )
-
             wf_3_task_2_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task2'
@@ -367,42 +349,45 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, wf_3_task_2_ex.state)
 
     def test_pause_resume_cascade_up_from_subworkflow(self):
-        workbook = """
+        wb_text = """
         version: '2.0'
+
         name: wb
+
         workflows:
           wf1:
             tasks:
               task1:
                 workflow: wf2
-                on-success:
-                  - task3
+                on-success: task3
+
               task2:
                 workflow: wf3
-                on-success:
-                  - task3
+                on-success: task3
+
               task3:
                 join: all
-                action: std.noop
+
           wf2:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
+
           wf3:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
         """
 
-        wb_service.create_workbook_v2(workbook)
+        wb_service.create_workbook_v2(wb_text)
 
         # Start workflow execution.
         wf_1_ex = self.engine.start_workflow('wb.wf1')
@@ -490,8 +475,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -509,8 +492,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -522,8 +503,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -556,8 +535,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -575,8 +552,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -588,8 +563,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -626,8 +599,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -645,8 +616,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -658,8 +627,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -692,8 +659,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -711,8 +676,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -724,8 +687,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -793,10 +754,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 name='task1'
             )
 
-            wf_2_task_1_action_exs = db_api.get_action_executions(
-                task_execution_id=wf_2_task_1_ex.id
-            )
-
             wf_2_task_2_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task2'
@@ -811,10 +768,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 name='task1'
             )
 
-            wf_3_task_1_action_exs = db_api.get_action_executions(
-                task_execution_id=wf_3_task_1_ex.id
-            )
-
             wf_3_task_2_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task2'
@@ -837,43 +790,46 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, wf_3_task_2_ex.state)
 
     def test_pause_resume_cascade_down_to_with_items_subworkflows(self):
-        workbook = """
+        wb_text = """
         version: '2.0'
+
         name: wb
+
         workflows:
           wf1:
             tasks:
               task1:
                 with-items: i in <% range(3) %>
                 workflow: wf2
-                on-success:
-                  - task3
+                on-success: task3
+
               task2:
                 workflow: wf3
-                on-success:
-                  - task3
+                on-success: task3
+
               task3:
                 join: all
-                action: std.noop
+
           wf2:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
+
           wf3:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
         """
 
-        wb_service.create_workbook_v2(workbook)
+        wb_service.create_workbook_v2(wb_text)
 
         # Start workflow execution.
         wf_1_ex = self.engine.start_workflow('wb.wf1')
@@ -1025,8 +981,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -1049,8 +1003,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[0].id
             )
 
-            wf_2_ex_1_task_execs = wf_2_ex_1.task_executions
-
             wf_2_ex_1_task_1_ex = self._assert_single_item(
                 wf_2_ex_1.task_executions,
                 name='task1'
@@ -1064,8 +1016,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[1].id
             )
 
-            wf_2_ex_2_task_execs = wf_2_ex_2.task_executions
-
             wf_2_ex_2_task_1_ex = self._assert_single_item(
                 wf_2_ex_2.task_executions,
                 name='task1'
@@ -1079,8 +1029,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[2].id
             )
 
-            wf_2_ex_3_task_execs = wf_2_ex_3.task_executions
-
             wf_2_ex_3_task_1_ex = self._assert_single_item(
                 wf_2_ex_3.task_executions,
                 name='task1'
@@ -1093,8 +1041,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the wf3 subworkflow execution.
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -1149,8 +1095,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -1173,8 +1117,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[0].id
             )
 
-            wf_2_ex_1_task_execs = wf_2_ex_1.task_executions
-
             wf_2_ex_1_task_1_ex = self._assert_single_item(
                 wf_2_ex_1.task_executions,
                 name='task1'
@@ -1188,8 +1130,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[1].id
             )
 
-            wf_2_ex_2_task_execs = wf_2_ex_2.task_executions
-
             wf_2_ex_2_task_1_ex = self._assert_single_item(
                 wf_2_ex_2.task_executions,
                 name='task1'
@@ -1203,8 +1143,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[2].id
             )
 
-            wf_2_ex_3_task_execs = wf_2_ex_3.task_executions
-
             wf_2_ex_3_task_1_ex = self._assert_single_item(
                 wf_2_ex_3.task_executions,
                 name='task1'
@@ -1217,8 +1155,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the wf3 subworkflow execution.
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -1292,8 +1228,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -1316,8 +1250,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[0].id
             )
 
-            wf_2_ex_1_task_execs = wf_2_ex_1.task_executions
-
             wf_2_ex_1_task_1_ex = self._assert_single_item(
                 wf_2_ex_1.task_executions,
                 name='task1'
@@ -1331,8 +1263,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[1].id
             )
 
-            wf_2_ex_2_task_execs = wf_2_ex_2.task_executions
-
             wf_2_ex_2_task_1_ex = self._assert_single_item(
                 wf_2_ex_2.task_executions,
                 name='task1'
@@ -1346,8 +1276,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[2].id
             )
 
-            wf_2_ex_3_task_execs = wf_2_ex_3.task_executions
-
             wf_2_ex_3_task_1_ex = self._assert_single_item(
                 wf_2_ex_3.task_executions,
                 name='task1'
@@ -1360,8 +1288,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the wf3 subworkflow execution.
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -1401,43 +1327,46 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, wf_3_task_1_action_exs[0].state)
 
     def test_pause_resume_cascade_up_from_with_items_subworkflow(self):
-        workbook = """
+        wb_text = """
         version: '2.0'
+
         name: wb
+
         workflows:
           wf1:
             tasks:
               task1:
                 with-items: i in <% range(3) %>
                 workflow: wf2
-                on-success:
-                  - task3
+                on-success: task3
+
               task2:
                 workflow: wf3
-                on-success:
-                  - task3
+                on-success: task3
+
               task3:
                 join: all
-                action: std.noop
+
           wf2:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
+
           wf3:
             tasks:
               task1:
                 action: std.async_noop
-                on-success:
-                  - task2
+                on-success: task2
+
               task2:
                 action: std.noop
         """
 
-        wb_service.create_workbook_v2(workbook)
+        wb_service.create_workbook_v2(wb_text)
 
         # Start workflow execution.
         wf_1_ex = self.engine.start_workflow('wb.wf1')
@@ -1782,8 +1711,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[2].id
             )
 
-            wf_2_ex_3_task_execs = wf_2_ex_3.task_executions
-
             wf_2_ex_3_task_1_ex = self._assert_single_item(
                 wf_2_ex_3.task_executions,
                 name='task1'
@@ -1796,8 +1723,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the wf3 subworkflow execution.
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -1877,8 +1802,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -1901,8 +1824,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[0].id
             )
 
-            wf_2_ex_1_task_execs = wf_2_ex_1.task_executions
-
             wf_2_ex_1_task_1_ex = self._assert_single_item(
                 wf_2_ex_1.task_executions,
                 name='task1'
@@ -1916,8 +1837,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[1].id
             )
 
-            wf_2_ex_2_task_execs = wf_2_ex_2.task_executions
-
             wf_2_ex_2_task_1_ex = self._assert_single_item(
                 wf_2_ex_2.task_executions,
                 name='task1'
@@ -1931,8 +1850,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
                 wf_1_task_1_action_exs[2].id
             )
 
-            wf_2_ex_3_task_execs = wf_2_ex_3.task_executions
-
             wf_2_ex_3_task_1_ex = self._assert_single_item(
                 wf_2_ex_3.task_executions,
                 name='task1'
@@ -1945,8 +1862,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the wf3 subworkflow execution.
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -1986,43 +1901,46 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, wf_3_task_1_action_exs[0].state)
 
     def test_pause_resume_cascade_up_from_subworkflow_pause_before(self):
-        workbook = """
+        wb_text = """
         version: '2.0'
+
         name: wb
+
         workflows:
-            wf1:
-                tasks:
-                    task1:
-                        workflow: wf2
-                        on-success:
-                            - task3
-                    task2:
-                        workflow: wf3
-                        on-success:
-                            - task3
-                    task3:
-                        join: all
-                        action: std.noop
-            wf2:
-                tasks:
-                    task1:
-                        action: std.noop
-                        on-success:
-                            - task2
-                    task2:
-                        pause-before: true
-                        action: std.async_noop
-            wf3:
-                tasks:
-                    task1:
-                        action: std.async_noop
-                        on-success:
-                            - task2
-                    task2:
-                        action: std.noop
+          wf1:
+            tasks:
+              task1:
+                workflow: wf2
+                on-success: task3
+
+              task2:
+                workflow: wf3
+                on-success: task3
+
+              task3:
+                join: all
+
+          wf2:
+            tasks:
+              task1:
+                action: std.noop
+                on-success: task2
+
+              task2:
+                pause-before: true
+                action: std.async_noop
+
+          wf3:
+            tasks:
+              task1:
+                action: std.async_noop
+                on-success: task2
+
+              task2:
+                action: std.noop
         """
 
-        wb_service.create_workbook_v2(workbook)
+        wb_service.create_workbook_v2(wb_text)
 
         # Start workflow execution.
         wf_1_ex = self.engine.start_workflow('wb.wf1')
@@ -2035,8 +1953,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -2054,8 +1970,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -2076,8 +1990,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
@@ -2114,8 +2026,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the parent workflow execution.
             wf_1_ex = self._assert_single_item(wf_execs, name='wb.wf1')
 
-            wf_1_task_execs = wf_1_ex.task_executions
-
             wf_1_task_1_ex = self._assert_single_item(
                 wf_1_ex.task_executions,
                 name='task1'
@@ -2133,8 +2043,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
             # Get objects for the subworkflow executions.
             wf_2_ex = self._assert_single_item(wf_execs, name='wb.wf2')
 
-            wf_2_task_execs = wf_2_ex.task_executions
-
             wf_2_task_1_ex = self._assert_single_item(
                 wf_2_ex.task_executions,
                 name='task1'
@@ -2155,8 +2063,6 @@ class SubworkflowPauseResumeTest(base.EngineTestCase):
 
             wf_3_ex = self._assert_single_item(wf_execs, name='wb.wf3')
 
-            wf_3_task_execs = wf_3_ex.task_executions
-
             wf_3_task_1_ex = self._assert_single_item(
                 wf_3_ex.task_executions,
                 name='task1'
