@@ -13,26 +13,30 @@
 #    limitations under the License.
 
 import copy
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 
 from mistral.actions import std_actions
+from mistral import config
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
 from mistral.services import workbooks as wb_service
 from mistral.services import workflows as wf_service
 from mistral.tests.unit import base as test_base
 from mistral.tests.unit.engine import base
-from mistral import utils
 from mistral.workflow import data_flow
 from mistral.workflow import states
 from mistral_lib import actions as actions_base
+from mistral_lib import utils
 
 # TODO(nmakhotkin) Need to write more tests.
 
 # Use the set_default method to set value otherwise in certain test cases
 # the change in value is not permanent.
-cfg.CONF.set_default('auth_enable', False, group='pecan')
+cfg.CONF.set_default('auth_enable', False, group=config.PECAN_GROUP)
+cfg.CONF.set_default('max_missed_heartbeats', 0,
+                     group=config.ACTION_HEARTBEAT_GROUP)
 
 WB = """
 ---
@@ -42,8 +46,6 @@ name: wb
 
 workflows:
   wf:
-    type: direct
-
     input:
      - names_info
 
@@ -64,8 +66,6 @@ name: wb
 
 workflows:
   wf:
-    type: direct
-
     input:
      - names_info
      - greeting
@@ -87,8 +87,6 @@ name: wb
 
 workflows:
   wf:
-    type: direct
-
     input:
      - arrayI
      - arrayJ
@@ -113,17 +111,13 @@ name: wb
 
 workflows:
   wf:
-    type: direct
-
     input:
-      - links
+      - items
 
     tasks:
       task1:
-        with-items: link in <% $.links %>
-        action: std.http url=<% $.link %>
-        publish:
-          result: <% task(task1) %>
+        with-items: item in <% $.items %>
+        action: std.async_noop
 """
 
 
@@ -136,14 +130,6 @@ WF_INPUT = {
 }
 
 
-WF_INPUT_URLS = {
-    'links': [
-        'http://google.com',
-        'http://openstack.org',
-        'http://google.com'
-    ]
-}
-
 WF_INPUT_ONE_ITEM = {
     'names_info': [
         {'name': 'Guy'}
@@ -165,21 +151,28 @@ class RandomSleepEchoAction(actions_base.Action):
 
 
 class WithItemsEngineTest(base.EngineTestCase):
-    def _assert_capacity(self, capacity, task_ex):
-        self.assertEqual(
-            capacity,
-            task_ex.runtime_context['with_items']['capacity']
-        )
+    @staticmethod
+    def _get_incomplete_action(task_ex_id):
+        with db_api.transaction():
+            task_ex = db_api.get_task_execution(task_ex_id)
+
+            return [e for e in task_ex.executions if not e.accepted][0]
 
     @staticmethod
-    def _get_incomplete_action(task_ex):
-        return [e for e in task_ex.executions if not e.accepted][0]
+    def _get_running_actions_count(task_ex_id):
+        with db_api.transaction():
+            task_ex = db_api.get_task_execution(task_ex_id)
+
+            return len(
+                [e for e in task_ex.executions if e.state == states.RUNNING]
+            )
 
     @staticmethod
-    def _get_running_actions_count(task_ex):
-        return len(
-            [e for e in task_ex.executions if e.state == states.RUNNING]
-        )
+    def _action_result_equals(action_ex_id, output):
+        with db_api.transaction():
+            a_ex = db_api.get_action_execution(action_ex_id)
+
+            return a_ex.output == output
 
     def test_with_items_simple(self):
         wb_service.create_workbook_v2(WB)
@@ -386,10 +379,16 @@ class WithItemsEngineTest(base.EngineTestCase):
         self.assertEqual(states.SUCCESS, task1_ex.state)
 
     def test_with_items_action_context(self):
+        # TODO(rakhmerov): Seems like the name of the test is not valid
+        # anymore since there's nothing related to action context in it.
+        # We need to revisit and refactor the entire module.
         wb_service.create_workbook_v2(WB_ACTION_CONTEXT)
 
         # Start workflow.
-        wf_ex = self.engine.start_workflow('wb.wf', wf_input=WF_INPUT_URLS)
+        wf_ex = self.engine.start_workflow(
+            'wb.wf',
+            wf_input={'items': [1, 2, 3]}
+        )
 
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
@@ -635,51 +634,54 @@ class WithItemsEngineTest(base.EngineTestCase):
             # Also initialize lazy collections.
             task_ex = wf_ex.task_executions[0]
 
-            self._assert_capacity(0, task_ex)
-            self.assertEqual(1, self._get_running_actions_count(task_ex))
+        self.assertEqual(1, self._get_running_actions_count(task_ex.id))
 
         # 1st iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("John")
         )
 
         # Wait till the delayed on_action_complete is processed.
-        # 1 is always there to periodically check WF completion.
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'John'})
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            self._assert_capacity(0, task_ex)
-            self.assertEqual(1, self._get_running_actions_count(task_ex))
+        self._await(lambda: self._get_running_actions_count(task_ex.id) == 1)
 
         # 2nd iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("Ivan")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'Ivan'})
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            self._assert_capacity(0, task_ex)
-            self.assertEqual(1, self._get_running_actions_count(task_ex))
+        self._await(lambda: self._get_running_actions_count(task_ex.id) == 1)
 
         # 3rd iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("Mistral")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) in (0, 1))
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'Mistral'})
+        )
 
         task_ex = db_api.get_task_execution(task_ex.id)
 
-        self._assert_capacity(1, task_ex)
-
         self.await_workflow_success(wf_ex.id)
 
         # Since we know that we can receive results in random order,
@@ -797,59 +799,53 @@ class WithItemsEngineTest(base.EngineTestCase):
 
             task_ex = wf_ex.task_executions[0]
 
-            running_cnt = self._get_running_actions_count(task_ex)
-
-        self._assert_capacity(0, task_ex)
-        self.assertEqual(2, running_cnt)
+        self.assertEqual(2, self._get_running_actions_count(task_ex.id))
 
         # 1st iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("John")
         )
 
         # Wait till the delayed on_action_complete is processed.
-        # 1 is always there to periodically check WF completion.
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'John'})
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            running_cnt = self._get_running_actions_count(task_ex)
-
-        self._assert_capacity(0, task_ex)
-        self.assertEqual(2, running_cnt)
+        self._await(lambda: self._get_running_actions_count(task_ex.id) == 2)
 
         # 2nd iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("Ivan")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'Ivan'})
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            running_cnt = self._get_running_actions_count(task_ex)
-
-        self._assert_capacity(0, task_ex)
-        self.assertEqual(2, running_cnt)
+        self._await(lambda: self._get_running_actions_count(task_ex.id) == 2)
 
         # 3rd iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("Mistral")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'Mistral'})
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            self._assert_capacity(1, task_ex)
-
-            incomplete_action = self._get_incomplete_action(task_ex)
+        incomplete_action = self._get_incomplete_action(task_ex.id)
 
         # 4th iteration complete.
         self.engine.on_action_complete(
@@ -857,12 +853,15 @@ class WithItemsEngineTest(base.EngineTestCase):
             actions_base.Result("Hello")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) in (0, 1))
+        self._await(
+            lambda: self._action_result_equals(
+                incomplete_action.id,
+                {'result': 'Hello'}
+            )
+        )
 
         task_ex = db_api.get_task_execution(task_ex.id)
 
-        self._assert_capacity(2, task_ex)
-
         self.await_workflow_success(wf_ex.id)
 
         # Since we know that we can receive results in random order,
@@ -949,27 +948,24 @@ class WithItemsEngineTest(base.EngineTestCase):
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
             task_ex = wf_ex.task_executions[0]
-            running_cnt = self._get_running_actions_count(task_ex)
 
-        self._assert_capacity(0, task_ex)
-        self.assertEqual(3, running_cnt)
+        self.assertEqual(3, self._get_running_actions_count(task_ex.id))
 
         # 1st iteration complete.
+        action_ex_id = self._get_incomplete_action(task_ex.id).id
+
         self.engine.on_action_complete(
-            self._get_incomplete_action(task_ex).id,
+            action_ex_id,
             actions_base.Result("John")
         )
 
         # Wait till the delayed on_action_complete is processed.
-        # 1 is always there to periodically check WF completion.
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(action_ex_id, {'result': 'John'})
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            self._assert_capacity(1, task_ex)
-
-            incomplete_action = self._get_incomplete_action(task_ex)
+        incomplete_action = self._get_incomplete_action(task_ex.id)
 
         # 2nd iteration complete.
         self.engine.on_action_complete(
@@ -977,14 +973,15 @@ class WithItemsEngineTest(base.EngineTestCase):
             actions_base.Result("Ivan")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) == 1)
+        self._await(
+            lambda:
+            self._action_result_equals(
+                incomplete_action.id,
+                {'result': 'Ivan'}
+            )
+        )
 
-        with db_api.transaction():
-            task_ex = db_api.get_task_execution(task_ex.id)
-
-            self._assert_capacity(2, task_ex)
-
-            incomplete_action = self._get_incomplete_action(task_ex)
+        incomplete_action = self._get_incomplete_action(task_ex.id)
 
         # 3rd iteration complete.
         self.engine.on_action_complete(
@@ -992,12 +989,16 @@ class WithItemsEngineTest(base.EngineTestCase):
             actions_base.Result("Mistral")
         )
 
-        self._await(lambda: len(db_api.get_delayed_calls()) in (0, 1))
+        self._await(
+            lambda:
+            self._action_result_equals(
+                incomplete_action.id,
+                {'result': 'Mistral'}
+            )
+        )
 
         task_ex = db_api.get_task_execution(task_ex.id)
 
-        self._assert_capacity(3, task_ex)
-
         self.await_workflow_success(wf_ex.id)
 
         with db_api.transaction():
@@ -1186,6 +1187,41 @@ class WithItemsEngineTest(base.EngineTestCase):
 
         self.assertEqual(states.SUCCESS, task1.state)
 
+    def test_with_items_env_in_with_items_expression(self):
+        wf_text = """---
+        version: "2.0"
+
+        wf:
+          tasks:
+            task1:
+              with-items: env_param in <% env().input_array %>
+              action: std.echo output=<% $.env_param %>
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow(
+            'wf',
+            env={'input_array': ['1', '2', '33']}
+        )
+
+        self.await_workflow_success(wf_ex.id, timeout=10)
+
+        with db_api.transaction():
+            # Note: We need to reread execution to access related tasks.
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task1 = self._assert_single_item(
+                wf_ex.task_executions,
+                name='task1'
+            )
+
+            result = data_flow.get_task_execution_result(task1)
+
+        self.assertListEqual(['1', '2', '33'], result)
+        self.assertEqual(states.SUCCESS, task1.state)
+
     def test_with_items_two_tasks_second_starts_on_success(self):
         wb_text = """---
         version: "2.0"
