@@ -13,13 +13,13 @@
 #    limitations under the License.
 
 import datetime
-import mock
+from unittest import mock
 
 from mistral.db.v2.sqlalchemy import api as db_api
 from mistral import exceptions as exc
 from mistral.expressions import jinja_expression as expr
 from mistral.tests.unit import base
-from mistral import utils
+from mistral_lib import utils
 
 DATA = {
     "server": {
@@ -71,7 +71,7 @@ WF_EXECS = [
 ]
 
 
-class JinjaEvaluatorTest(base.BaseTest):
+class JinjaEvaluatorTest(base.DbTestCase):
     def setUp(self):
         super(JinjaEvaluatorTest, self).setUp()
 
@@ -79,6 +79,7 @@ class JinjaEvaluatorTest(base.BaseTest):
 
     def test_expression_result(self):
         res = self._evaluator.evaluate('_.server', DATA)
+
         self.assertEqual({
             'id': '03ea824a-aa24-4105-9131-66c48ae54acf',
             'name': 'cloud-fedora',
@@ -86,9 +87,11 @@ class JinjaEvaluatorTest(base.BaseTest):
         }, res)
 
         res = self._evaluator.evaluate('_.server.id', DATA)
+
         self.assertEqual('03ea824a-aa24-4105-9131-66c48ae54acf', res)
 
         res = self._evaluator.evaluate("_.server.status == 'ACTIVE'", DATA)
+
         self.assertTrue(res)
 
     def test_select_result(self):
@@ -96,7 +99,9 @@ class JinjaEvaluatorTest(base.BaseTest):
             '_.servers|selectattr("name", "equalto", "ubuntu")',
             SERVERS
         )
+
         item = list(res)[0]
+
         self.assertEqual({'name': 'ubuntu'}, item)
 
     def test_function_string(self):
@@ -104,8 +109,11 @@ class JinjaEvaluatorTest(base.BaseTest):
         self.assertEqual('3', self._evaluator.evaluate('_|string', 3))
 
     def test_function_len(self):
-        self.assertEqual(3,
-                         self._evaluator.evaluate('_|length', 'hey'))
+        self.assertEqual(
+            3,
+            self._evaluator.evaluate('_|length', 'hey')
+        )
+
         data = [{'some': 'thing'}]
 
         self.assertEqual(
@@ -184,10 +192,12 @@ class JinjaEvaluatorTest(base.BaseTest):
 
     def test_function_env(self):
         ctx = {'__env': 'some'}
+
         self.assertEqual(ctx['__env'], self._evaluator.evaluate('env()', ctx))
 
     def test_filter_env(self):
         ctx = {'__env': 'some'}
+
         self.assertEqual(ctx['__env'], self._evaluator.evaluate('_|env', ctx))
 
     @mock.patch('mistral.db.v2.api.get_task_executions')
@@ -196,6 +206,7 @@ class JinjaEvaluatorTest(base.BaseTest):
                                                 task_executions):
         task = mock.MagicMock(return_value={})
         task_executions.return_value = [task]
+
         ctx = {
             '__task_execution': None,
             '__execution': {
@@ -216,7 +227,7 @@ class JinjaEvaluatorTest(base.BaseTest):
             'type': task.type,
             'workflow_execution_id': task.workflow_execution_id,
             'created_at': task.created_at.isoformat(' '),
-            'updated_at': task.updated_at.isoformat(' ')
+            'updated_at': task.updated_at.isoformat(' '),
         }, result)
 
     @mock.patch('mistral.db.v2.api.get_task_executions')
@@ -320,7 +331,8 @@ class JinjaEvaluatorTest(base.BaseTest):
             'input': wf_ex.input,
             'params': wf_ex.params,
             'created_at': wf_ex.created_at.isoformat(' '),
-            'updated_at': wf_ex.updated_at.isoformat(' ')
+            'updated_at': wf_ex.updated_at.isoformat(' '),
+            'root_execution_id': wf_ex.root_execution_id
         }, result)
 
     def test_executions(self):
@@ -458,7 +470,8 @@ class JinjaEvaluatorTest(base.BaseTest):
             'input': wf_ex.input,
             'params': wf_ex.params,
             'created_at': wf_ex.created_at.isoformat(' '),
-            'updated_at': wf_ex.updated_at.isoformat(' ')
+            'updated_at': wf_ex.updated_at.isoformat(' '),
+            'root_execution_id': wf_ex.root_execution_id
         }, result)
 
 
