@@ -1,4 +1,5 @@
 # Copyright 2014 - Mirantis, Inc.
+# Copyright 2020 Nokia Software.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -15,17 +16,16 @@
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
+from mistral.exceptions import DBEntityNotFoundError
 from mistral.lang import parser as spec_parser
 from mistral.services import actions as action_service
 from mistral.tests.unit import base
-from mistral import utils
-
+from mistral_lib import utils
 
 # Use the set_default method to set value otherwise in certain test cases
 # the change in value is not permanent.
 cfg.CONF.set_default('auth_enable', False, group='pecan')
 
-
 ACTION_LIST = """
 ---
 version: '2.0'
@@ -54,6 +54,8 @@ action1:
     result: $
 """
 
+NAMESPACE = 'test_namespace'
+
 
 class ActionServiceTest(base.DbTestCase):
     def setUp(self):
@@ -78,14 +80,35 @@ class ActionServiceTest(base.DbTestCase):
 
         # Action 2.
         action2_db = self._assert_single_item(db_actions, name='action2')
+        self.assertEqual('', action2_db.namespace)
         action2_spec = spec_parser.get_action_spec(action2_db.spec)
 
         self.assertEqual('action2', action2_spec.get_name())
         self.assertEqual('std.echo', action1_spec.get_base())
         self.assertDictEqual({'output': 'Hey'}, action2_spec.get_base_input())
 
+    def test_create_actions_in_namespace(self):
+        db_actions = action_service.create_actions(ACTION_LIST,
+                                                   namespace=NAMESPACE)
+
+        self.assertEqual(2, len(db_actions))
+
+        action1_db = self._assert_single_item(db_actions, name='action1')
+        self.assertEqual(NAMESPACE, action1_db.namespace)
+
+        action2_db = self._assert_single_item(db_actions, name='action2')
+        self.assertEqual(NAMESPACE, action2_db.namespace)
+
+        self.assertRaises(
+            DBEntityNotFoundError,
+            db_api.get_action_definition,
+            name='action1',
+            namespace=''
+        )
+
     def test_update_actions(self):
-        db_actions = action_service.create_actions(ACTION_LIST)
+        db_actions = action_service.create_actions(ACTION_LIST,
+                                                   namespace=NAMESPACE)
 
         self.assertEqual(2, len(db_actions))
 
@@ -97,7 +120,8 @@ class ActionServiceTest(base.DbTestCase):
         self.assertDictEqual({'output': 'Hi'}, action1_spec.get_base_input())
         self.assertDictEqual({}, action1_spec.get_input())
 
-        db_actions = action_service.update_actions(UPDATED_ACTION_LIST)
+        db_actions = action_service.update_actions(UPDATED_ACTION_LIST,
+                                                   namespace=NAMESPACE)
 
         # Action 1.
         action1_db = self._assert_single_item(db_actions, name='action1')
@@ -112,3 +136,29 @@ class ActionServiceTest(base.DbTestCase):
             action1_spec.get_input().get('param1'),
             utils.NotDefined
         )
+
+        self.assertRaises(
+            DBEntityNotFoundError,
+            action_service.update_actions,
+            UPDATED_ACTION_LIST,
+            namespace=''
+        )
+
+    def test_delete_action(self):
+
+        # Create action.
+        action_service.create_actions(ACTION_LIST, namespace=NAMESPACE)
+
+        action = db_api.get_action_definition('action1', namespace=NAMESPACE)
+        self.assertEqual(NAMESPACE, action.get('namespace'))
+        self.assertEqual('action1', action.get('name'))
+
+        # Delete action.
+        db_api.delete_action_definition('action1', namespace=NAMESPACE)
+
+        self.assertRaises(
+            DBEntityNotFoundError,
+            db_api.get_action_definition,
+            name='action1',
+            namespace=NAMESPACE
+        )
