@@ -235,7 +235,7 @@ def get_all(list_cls, cls, get_all_function, get_function,
     return list_cls.convert_with_links(
         rest_resources,
         limit,
-        pecan.request.host_url,
+        pecan.request.application_url,
         sort_keys=','.join(sort_keys),
         sort_dirs=','.join(sort_dirs),
         fields=','.join(fields) if fields else '',
@@ -267,3 +267,15 @@ def create_db_retry_object():
 
 def rest_retry_on_db_error(func):
     return db_utils.retry_on_db_error(func, create_db_retry_object())
+
+
+def load_deferred_fields(ex, fields):
+    if not ex:
+        return ex
+
+    # We need to refer lazy-loaded fields explicitly in
+    # order to make sure that they are correctly loaded.
+    for f in fields:
+        hasattr(ex, f)
+
+    return ex
